<?php
namespace App\Service;
class Branch {

  private $branchFactory;
  private $dateUtil;

  public function __construct(\App\Factory\Branch $branchFactory,
                              \App\Util\DateUtil $dateUtil) {
    $this->branchFactory = $branchFactory;
    $this->dateUtil      = $dateUtil;
  }

  public function getUnits() {
    $blockedStores = array();
    $units = $this->branchFactory->getUnits();

    if (!empty($units)) {
        foreach ($units as $unit) {
            $blockedStores[] = $unit;
            $this->processBlockedStores($unit, $blockedStores);
            $this->processAvailableStores($unit, $blockedStores);
        }
    }
    return $blockedStores;
}

private function processBlockedStores($unit, &$blockedStores) {
  $params = array('CDFILIAL' => $unit['CDFILIALGRID']);
  $stores = $this->branchFactory->getBlockedStores($params);
  $newArray = array();
  $currentFilialGrid = null;

  foreach ($stores as $index => $store) {
      $this->prepareStoreData($store, $unit);

      if ($store['CDFILIALGRID'] !== $currentFilialGrid) {
          $newArray = array(
              'CDFILIALGRID' => $store['CDFILIALGRID'],
              'CDFILIAL'     => $store['CDFILIAL'] . $index . $unit['CDFILIAL'],
              'FILIAL'       => $unit['FILIAL'],
              'NMLOJA'       => $store['NMLOJA'],
              'BLOCK'        => 'ISFALSE',
              'PARENTID'     => isset($unit['CDFILIAL']) ? $unit['CDFILIAL'] : null,
          );

          $blockedStores[] = $newArray;
          $currentFilialGrid = $store['CDFILIALGRID'];
      }

        $storeArray = array(
          'CDFILIALGRID'     => $store['CDFILIALGRID'],
          'CDFILIAL'         => $store['CDFILIAL'] . $index,
          'NMLOJA'           => $store['NMLOJA'],
          'NMOPERADOR'       => $store['NMOPERADOR'],
          'FILIAL'           => $store['FILIAL'],
          'DTHRBLOQUEIO'     => $store['DTHRBLOQUEIO'],
          'NRDIASEMANABLOQ'  => $store['NRDIASEMANABLOQ'],
          'LABELNRDIASEMANA' => $store['LABELNRDIASEMANA'],
          'PARENTID'         => isset($newArray['CDFILIAL']) ? $newArray['CDFILIAL'] : $unit['CDFILIAL'],
          'BLOCK'            => 'ISTRUE',
        );

      $blockedStores[] = $storeArray;
  }
}

  private function processAvailableStores($unit, &$blockedStores) {
      $availableStoresByUnit = $this->branchFactory->getAvailableStoresByUnit(array('CDFILIAL' => $unit['CDFILIALGRID']));

      foreach ($availableStoresByUnit as $availableStore) {
          $availableStore['PARENTID'] = $unit['CDFILIAL'];
          $availableStore['FILIAL']   = $unit['FILIAL'];
          $blockedStores[] = $availableStore;
      }
  }

  private function prepareStoreData(&$store, $unit) {
      $store['LABELNRDIASEMANA'] = $this->dateUtil->getWeekDay($store['NRDIASEMANABLOQ']);
  }

  public function blockStores($insertParams) {
    $this->removeOtherDaysStore($insertParams);
    $results = array();
    if (is_array($insertParams['NRDIASEMANABLOQ'])) {
        foreach ($insertParams['CDLOJA'] as $cdLoja) {
            foreach ($insertParams['NRDIASEMANABLOQ'] as $nrDias) {
                $params = array(
                    'CDFILIAL' => $insertParams['CDFILIAL'],
                    'CDLOJA' => $cdLoja,
                    'CDOPERADOR' => $insertParams['CDOPERADOR'],
                    'NRDIASEMANABLOQ' => $nrDias
                );
              $this->branchFactory->blockStores($params);
            }
        }
    }
  }

  public function unlockStores($unlockStores) {
    foreach ($unlockStores as $store) {
      $this->branchFactory->unlockStores($store);
    }
  }

  private function removeOtherDaysStore($deleteOtherDays) {
    foreach ($deleteOtherDays['CDLOJA'] as $cdLoja) {
      $params = array(
        'CDFILIAL'        => $deleteOtherDays['CDFILIAL'],
        'CDLOJA'          => $cdLoja,
      );
      $this->branchFactory->removeOtherDaysStore($params);
    }
  }

  public function getBlockedProductUnits($search) {
    $blockedProducts = array();
    $units = $this->branchFactory->getUnits($search);
    $count = 0;
    if(!empty($units)) {
      foreach($units as $unit) {
        $count ++;
        $params = array('CDFILIAL' => $unit['CDFILIALGRID']);
        $stores = $this->branchFactory->getStores($params);
        array_push($blockedProducts, $unit);
        foreach($stores as $store) {
          $store['PARENTID'] = $unit['CDFILIAL'];
          $store['PARENTFILIAL'] = $unit['CDFILIALGRID'];
          array_push($blockedProducts, $store);
          $params['CDFILIALGRID'] = $store['CDFILIALGRID'];
          $blockedProductsList = $this->branchFactory->getBlockedProduct($params);
          foreach($blockedProductsList as $blockedProduct) {
            $blockedProduct['CDFILIAL'] = $blockedProduct['CDFILIALGRID'] . $store['CDFILIALGRID'] . $blockedProduct['NRDIASEMANABLOQ'];
            $blockedProduct['PARENTID'] = $store['CDFILIAL'];
            $blockedProduct['PARENTFILIAL'] = $unit['CDFILIALGRID'];
            $blockedProduct['PARENTSTORE'] = $store['CDFILIALGRID'];
            $blockedProduct['LABELNRDIASEMANA'] = $this->dateUtil->getWeekDay($blockedProduct['NRDIASEMANABLOQ']);
            array_push($blockedProducts, $blockedProduct);
          }
        }
      }
      return [
        'data'  => $blockedProducts,
        'total' => $count
      ];
    }
    return array();
  }

  public function getAvailableProductsByUnit($params) {
    return $this->branchFactory->getAvailableProductsByUnit($params);
  }

  public function blockProducts($insertParams) {
    if (is_array($insertParams['NRDIASEMANABLOQ'])) {
        foreach ($insertParams['CDPRODUTO'] as $product) {
            foreach ($insertParams['NRDIASEMANABLOQ'] as $nrDias) {
              $params = array(
                  'CDFILIAL' => $insertParams['CDFILIAL'],
                  'CDLOJA' => $insertParams['CDLOJA'],
                  'CDPRODUTO' => $product,
                  'CDOPERADOR' => $insertParams['CDOPERADOR'],
                  'NRDIASEMANABLOQ' => $nrDias
              );
              $this->branchFactory->blockProducts($params);
            }
        }
    } elseif (is_string($insertParams['NRDIASEMANABLOQ'])) {
        foreach ($insertParams['CDPRODUTO'] as $product) {
          $params = array(
            'CDFILIAL'        => $insertParams['CDFILIAL'],
            'CDLOJA'          => $insertParams['CDLOJA'],
            'CDPRODUTO' => $product,
            'CDOPERADOR'      => $insertParams['CDOPERADOR'],
            'NRDIASEMANABLOQ' => $insertParams['NRDIASEMANABLOQ']
        );
        $this->branchFactory->blockProducts($params);
      }
    }
  }

  public function unblockProducts($deleteParams) {
    foreach ($deleteParams as $product) {
      if(!empty($product['CDLOJA'])){
        $params = array(
          'CDFILIAL' => $product['CDFILIAL'],
          'CDLOJA' => $product['CDLOJA'],
          'CDPRODUTO' => $product['CDPRODUTO'],
          'NRDIASEMANABLOQ' => $product['NRDIASEMANABLOQ'],
        );
        $this->branchFactory->unblockProducts($params);
      }
    }
  }

  public function getBranchs() {
		return $this->branchFactory->getBranchs();
	}
}
